/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Contreras
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.ccm.deploytool;

public class Connector {

    private DeploymentProcess _deployer;

    private java.util.Hashtable connections;

    public Connector (DeploymentProcess deployer)
	throws Exception {

	// Attaching to the deployment process
    this._deployer=deployer;

	// initializing the connections hashtable
    connections = new java.util.Hashtable();

	// Getting the assembly descriptor
    org.objectweb.ccm.descriptor.componentassembly.ComponentassemblyBean assemblyObject = _deployer.getAssemblyObject();

	// feeding the connections hashtable with the found connectinterfaces
    java.util.List connectInterfacesList
        = _deployer.getAssemblyObject().getConnections().getConnectinterfaceList();
    for (java.util.Iterator i = connectInterfacesList.iterator() ; i.hasNext() ; ) {
        org.objectweb.ccm.descriptor.componentassembly.ConnectinterfaceBean connectInterface
        = (org.objectweb.ccm.descriptor.componentassembly.ConnectinterfaceBean) i.next();

        connections.put(connectInterface,new Boolean (false));
	}

    //feeding the connections hashtable with the found connectevents
	java.util.List connectEventsList
	    = assemblyObject.getConnections().getConnecteventList();
	for (java.util.Iterator i = connectEventsList.iterator() ; i.hasNext() ; ) {
	    org.objectweb.ccm.descriptor.componentassembly.ConnecteventBean connectEvent
		= (org.objectweb.ccm.descriptor.componentassembly.ConnecteventBean) i.next();

	    connections.put(connectEvent, new Boolean(false));
	}

//	System.out.println("Connector is ready...");

    }

    private void doConnectInterface (org.objectweb.ccm.descriptor.componentassembly.ConnectinterfaceBean connectInterface) {

	if ( ( (Boolean)connections.get(connectInterface) ).booleanValue() && true) {

	    // building of the provide request that will be done
	    org.omg.CORBA.Request request
	    = ( (org.omg.Components.CCMObject)
		_deployer.getPartition().get(connectInterface.getProvidesport().getComponentinstantiationref().getIdref())
		)._request("provide_" + connectInterface.getProvidesport().getProvidesidentifier().getValue());

	    // setting of the return type (it provides a CORBA object)
	    request.set_return_type(org.omg.CORBA.ObjectHelper.type());

        request.invoke();

        // getting the provided object
        org.omg.CORBA.Object providedObject
		  = request.return_value().extract_Object();

	    // getting the port converned in the user component
	    org.omg.Components.CCMObject userConcerned
		= ( (org.omg.Components.CCMObject)
		    _deployer.getPartition().get(connectInterface.getUsesport().getComponentinstantiationref().getIdref())
		    );

	    // building the request on the port
	    request
		= userConcerned._request("connect_" + connectInterface.getUsesport().getUsesidentifier().getValue());

	    request.add_in_arg().insert_Object(providedObject);

/*
	    System.out.println(
		"connecting "
		+ connectInterface.getUsesport().getUsesidentifier().getValue()
		+ " to "
		+ connectInterface.getProvidesport().getProvidesidentifier().getValue()
		);
*/
	    request.invoke();
	}else{
//	    System.out.println("only 1 present, we'll wait");
	    connections.put(connectInterface, new Boolean(true));
	}
    }

    private void doConnectEvent (org.objectweb.ccm.descriptor.componentassembly.ConnecteventBean connectEvent) {

	if ( ( (Boolean)connections.get(connectEvent) ).booleanValue() && true) {
	
	    // building the request that will get the consumed event
	    org.omg.CORBA.Request request
              = ((org.omg.Components.CCMObject)
		    _deployer
		      .getPartition()
		      .get(
		        connectEvent
		          .getConsumesport()
			  .getComponentinstantiationref()
			  .getIdref()
		      )
                )._request(
		   "get_consumer_"
		   + connectEvent
		       .getConsumesport()
		       .getConsumesidentifier()
		       .getValue()
		 );

	    // this request will return a CORBA object
        request.set_return_type(org.omg.CORBA.ObjectHelper.type());

        request.invoke();

	    // getting the returned resource
        org.omg.CORBA.Object consumedObject
		= request.return_value().extract_Object();

	    // initiating the subscriber
	    org.omg.Components.CCMObject subscriberConcerned
	      = ((org.omg.Components.CCMObject)
		    _deployer
		      .getPartition()
		      .get(connectEvent
		             .getPublishesport()
			     .getComponentinstantiationref()
			     .getIdref()
		      )
		);

	    // building of the subscribe request
	    request
	      = subscriberConcerned
	          ._request(
		    "subscribe_"
		    + connectEvent
		        .getPublishesport()
			.getPublishesidentifier()
			.getValue()
		  );

	    // feeding the request with what is consumed
	    request.add_in_arg().insert_Object(consumedObject);
	    
/*
   System.out.println(
		"connecting "
		+ connectEvent.getPublishesport().getPublishesidentifier().getValue()
		+ " to "
		+ connectEvent.getConsumesport().getConsumesidentifier().getValue()
		);
*/

	    request.invoke();
	}else{
	    connections.put(connectEvent, new Boolean(true));
	}
    }

    private java.util.List searchForConnections (String componentId, String connectionType) {
	// this method search the connections of connectionType
	// and returns those where it can find the componentId

	java.util.List returnedList = new java.util.LinkedList();

	if (connectionType.equals("connectinterface")){
	    java.util.List connectInterfacesList
		= _deployer.getAssemblyObject().getConnections().getConnectinterfaceList();
	    for (java.util.Iterator i = connectInterfacesList.iterator() ; i.hasNext() ; ) {
		org.objectweb.ccm.descriptor.componentassembly.ConnectinterfaceBean connectInterface
		    = (org.objectweb.ccm.descriptor.componentassembly.ConnectinterfaceBean) i.next();
		if (componentId.equals(connectInterface.getProvidesport().getComponentinstantiationref().getIdref())) {
		    //System.out.println("component in a connectinterface");
		    returnedList.add(connectInterface);
		}
		if (componentId.equals(connectInterface.getUsesport().getComponentinstantiationref().getIdref())) {
		    returnedList.add(connectInterface);
		}
	    }
	}
	if (connectionType.equals("connectevent")){
	    java.util.List connectEventsList
		= _deployer.getAssemblyObject().getConnections().getConnecteventList();
	    for (java.util.Iterator i = connectEventsList.iterator() ; i.hasNext() ; ) {
		org.objectweb.ccm.descriptor.componentassembly.ConnecteventBean connectEvent
		    = (org.objectweb.ccm.descriptor.componentassembly.ConnecteventBean) i.next();
		if (componentId.equals(connectEvent.getConsumesport().getComponentinstantiationref().getIdref())) {
		    returnedList.add(connectEvent);
		}
		if (componentId.equals(connectEvent.getPublishesport().getComponentinstantiationref().getIdref())) {
		    returnedList.add(connectEvent);
		}
	    }
	}
    return returnedList;
    }

    public void componentReady (String componentId) {
	// this method is called by the partitioner once a component is ready
	// it looks for connections where the component is involved
	// and try to do it

    java.util.List concernedConnections = searchForConnections(componentId, "connectinterface");
	
	for (java.util.Iterator i = concernedConnections.iterator() ; i.hasNext() ; ) {
	    org.objectweb.ccm.descriptor.componentassembly.ConnectinterfaceBean connectInterface
		= (org.objectweb.ccm.descriptor.componentassembly.ConnectinterfaceBean) i.next();

	    doConnectInterface(connectInterface);
	}    

	concernedConnections = searchForConnections(componentId, "connectevent");
	
	for (java.util.Iterator i = concernedConnections.iterator() ; i.hasNext() ; ) {
	    org.objectweb.ccm.descriptor.componentassembly.ConnecteventBean connectEvent
		= (org.objectweb.ccm.descriptor.componentassembly.ConnecteventBean) i.next();
	    
	    doConnectEvent(connectEvent);
	}

    }

}
