/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Contreras
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.ccm.deploytool;

public class Partition extends java.util.Hashtable {

    private DeploymentProcess _deployer;
    
    public Partition (DeploymentProcess deployer)
    {

	this._deployer = deployer;
    }
    
    public void operate()
	throws Exception 
    {
    	org.objectweb.ccm.descriptor.componentassembly.ComponentassemblyBean assemblyObject = _deployer.getAssemblyObject();

        // homeplacements processing
	String destination = null;
	java.util.List placementsList 
	    = assemblyObject.getPartitioning().getHomeplacementList();
	processHomeplacementList(placementsList, destination);
	java.util.List collocationList = assemblyObject.getPartitioning().getProcesscollocationList();
	for (java.util.Iterator i = collocationList.iterator() ; i.hasNext() ; ) {
	    org.objectweb.ccm.descriptor.componentassembly.Processcollocation collocation
		= (org.objectweb.ccm.descriptor.componentassembly.Processcollocation) i.next();
	    if (collocation.getDestination() != null)
		destination = collocation.getDestination().getValue();

	    processHomeplacementList(
		collocation.getHomeplacementList()
		, destination);
	}
	collocationList = assemblyObject.getPartitioning().getHostcollocationList();
	for (java.util.Iterator i = collocationList.iterator() ; i.hasNext() ; ) {
	    org.objectweb.ccm.descriptor.componentassembly.Hostcollocation collocation
		= (org.objectweb.ccm.descriptor.componentassembly.Hostcollocation) i.next();
	    if (collocation.getDestination() != null)
		destination = collocation.getDestination().getValue();

	    processHomeplacementList(
		collocation.getHomeplacementList()
		, destination);
	}
    }
    
    private void processHomeplacementList ( java.util.List placementsList, String destination )
    throws Exception{

	for (java.util.Iterator i = placementsList.iterator() ; i.hasNext() ; ) {
	    org.objectweb.ccm.descriptor.componentassembly.HomeplacementBean homePlacement
	        = (org.objectweb.ccm.descriptor.componentassembly.HomeplacementBean) i.next();

	    // deploying the home
	    org.omg.Components.CCMHome homeDeployed = deploy_home(gatherHomeInfo(homePlacement, destination) );

	    // saving the reference in the zeus object
	    ( (org.objectweb.ccm.descriptor.componentassembly.HomeplacementDeployInfo)
		homePlacement.getExtensionManager().getExtensionByName("DeployInfo").getInstance() ).setCorbaReference(homeDeployed);

	    // components instantiations
	    java.util.List instantiationsList = homePlacement.getComponentinstantiationList();
	    for (java.util.Iterator ii = instantiationsList.iterator() ; ii.hasNext() ; ) {
		org.objectweb.ccm.descriptor.componentassembly.ComponentinstantiationBean instanceToSet
		    = (org.objectweb.ccm.descriptor.componentassembly.ComponentinstantiationBean) ii.next();

		org.omg.Components.CCMObject objectDeployed = deploy_and_set_component(gatherComponentInstanceInfo(instanceToSet), homeDeployed);

		((org.objectweb.ccm.descriptor.componentassembly.ComponentinstantiationDeployInfo)
		instanceToSet.getExtensionManager().getExtensionByName("DeployInfo").getInstance() ).setCorbaReference(objectDeployed);

		this.put(
		    instanceToSet.getId(),
		    ((org.objectweb.ccm.descriptor.componentassembly.ComponentinstantiationDeployInfo)	
			instanceToSet.getExtensionManager().getExtensionByName("DeployInfo").getInstance() ).getCorbaReference()
		    );

		_deployer.getConnector().componentReady(instanceToSet.getId());
	    }
	}
    }

    private  java.util.Hashtable gatherHomeInfo (
	org.objectweb.ccm.descriptor.componentassembly.HomeplacementBean homePlacement, String destination)
    throws Exception {

	java.util.Hashtable homeInfos = new java.util.Hashtable();
	
	if (destination != null) {
		homeInfos.put(
		    "destination"
		    , destination);
	}else{
	    if (homePlacement.getDestination().getValue() != null) {
		homeInfos.put(
		    "destination"
		    , homePlacement.getDestination().getValue() );
	    }else{
		System.out.println("WARNING, NO DESTINATION FOUND");
	    }
	}

	// getting the concerned softpkg descriptor
	java.util.List filesList = _deployer.getAssemblyObject().getComponentfiles().getComponentfileList();
	for (java.util.Iterator i = filesList.iterator() ; i.hasNext() ; ) {
	
	    org.objectweb.ccm.descriptor.componentassembly.ComponentfileBean componentFile
		= (org.objectweb.ccm.descriptor.componentassembly.ComponentfileBean) i.next();
		
	    if ( homePlacement.getComponentfileref().getIdref().equals (componentFile.getId()) ) {
		// this is the CSD we need

		java.io.File softpkgFile = new java.io.File ("./META-INF/" + componentFile.getFileinarchive().getName());

		//getting the object from softpkg descriptor
		org.objectweb.ccm.descriptor.softpkg.SoftpkgBean softpkgObject
		    = org.objectweb.ccm.descriptor.softpkg.SoftpkgImplBean.unmarshalBean(softpkgFile);
		
		org.objectweb.ccm.descriptor.softpkg.ImplementationBean implemToInstall
		    = (org.objectweb.ccm.descriptor.softpkg.ImplementationBean) softpkgObject.getImplementationList().get(0);
		
		homeInfos.put( 
		    "implemId"
		    , implemToInstall.getId()
		    );
		
		homeInfos.put(
		    "archiveFile"
		    , ((org.objectweb.ccm.descriptor.softpkg.CodeBean)
			implemToInstall.getCodeList().get(0)).getFileinarchive().getName()
		    );
		
		homeInfos.put(
		    "entryPoint"
		    , ((org.objectweb.ccm.descriptor.softpkg.CodeBean)
			implemToInstall.getCodeList().get(0)).getEntrypoint().getValue()
		    );
	    }
	}
	
	return (homeInfos);
	
    }

    private  org.omg.Components.CCMHome deploy_home (java.util.Hashtable homeInfos)
    throws Exception {
	
	// Obtain the component servers.
        org.omg.CosNaming.NameComponent[] ncomp =
                             new org.omg.CosNaming.NameComponent[1];

        ncomp[0] = new org.omg.CosNaming.NameComponent((String)homeInfos.get("destination"), "");

	org.omg.CORBA.Object obj = _deployer.getNc().resolve(ncomp);

        org.objectweb.ccm.Deployment.Server server =
            org.objectweb.ccm.Deployment.ServerHelper.narrow(obj);

        // Obtain the home factories and archive installators.
        org.omg.Components.Deployment.ComponentServer server_cs
	    = server.provide_component_server();
        org.omg.Components.Deployment.ComponentInstallation server_inst
	    = server.provide_install();
		
	// Install archives.
    	System.out.println(
	    "Installing archive "
	    + homeInfos.get("archiveFile")
	    + "...");
    	
	server_inst.install(
	    (String)homeInfos.get("implemId")
	    , (String)homeInfos.get("archiveFile")
	    );

	// Install some plugins already provided by OpenCCM
	server_inst.install("openccm_plugins", "./OpenCCM_Plugins.jar");
		
    	org.omg.Components.ConfigValue[] config = new org.omg.Components.ConfigValue[0];

    	org.omg.Components.Deployment.Container server_cont
	    = server_cs.create_container(config);

    	// Create homes.
    	System.out.println("Instantiating homes...");

    	// install homes with this container configuration.
    	org.omg.Components.CCMHome homePlaced = server_cont.install_home(
	    (String)homeInfos.get("implemId")
	    , (String)homeInfos.get("entryPoint")
	    , config
	    );
	    
	return (homePlaced);

    }

    private  java.util.Hashtable gatherComponentInstanceInfo (
	org.objectweb.ccm.descriptor.componentassembly.ComponentinstantiationBean instanceToSet)
    throws Exception {

	java.util.Hashtable instanceInfos = new java.util.Hashtable();

	instanceInfos.put(
	    "instanceId"
	    , instanceToSet.getId()
	    );

	java.util.List storedPropertiesList = new java.util.LinkedList();

	if ( instanceToSet.getComponentproperties() != null ) {
	    java.io.File propertiesFile = new java.io.File ("./META-INF/" + instanceToSet.getComponentproperties().getFileinarchive().getName() );

	    //getting the object from property descriptor
	    org.objectweb.ccm.descriptor.properties.PropertiesBean propertiesObject
		= org.objectweb.ccm.descriptor.properties.PropertiesImplBean.unmarshalBean(propertiesFile);

	    java.util.List propertiesList = propertiesObject.getSimpleList();
	    
	    for (java.util.Iterator i = propertiesList.iterator() ; i.hasNext() ; ) {
		
		org.objectweb.ccm.descriptor.properties.SimpleBean property
		    = (org.objectweb.ccm.descriptor.properties.SimpleBean) i.next();
		
		String[] name_type_value = {
		    property.getName()
		    , property.getType()
		    , property.getValue().getValue()
		    };
		
		storedPropertiesList.add(name_type_value);
	    }
	    
	}
	
	instanceInfos.put(
	    "propertiesList"
	    , storedPropertiesList
	    );
	
	return (instanceInfos);
    }

    private org.omg.Components.CCMObject deploy_and_set_component(
	java.util.Hashtable instanceInfos
	, org.omg.Components.CCMHome homeDeployed)
    throws Exception{

	org.omg.Components.CCMObject compoCreated = org.omg.Components.KeylessCCMHomeHelper.narrow( homeDeployed).create_component();

	// Configure components.

	for (java.util.Iterator i = ((java.util.List)instanceInfos.get("propertiesList")).iterator() ; i.hasNext() ; ) {

	    String[] name_type_value = (String[]) i.next();

	    applyProperty(compoCreated, name_type_value);
	}
	
	return (compoCreated);
	
    }

    private  void applyProperty (
	org.omg.Components.CCMObject component,
	String[] name_type_value) {

	org.omg.CORBA.Request request = component._request("_set_" + name_type_value[0] );
	if (name_type_value[1].equals( "string")) {
	    request.add_in_arg().insert_string(name_type_value[2]);
	}

	request.invoke();
    
    }

}
