/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Christophe Demarey.
Contributor(s): .

====================================================================*/

package org.objectweb.util.launcher;

/**
 * URL Class Loader.
 * This loader overwrite URL class loader dafult behaviour.
 * It searchs class first in specified URLs and last in the parent class loader.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class XBootClassLoader
     extends java.net.URLClassLoader
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public XBootClassLoader()
    {
        super(new java.net.URL[0]);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Loads the class with the specified name.
     *
     * @param name - The name of the class to be loaded. Must not be null.
     * @param resolve - if true then resolve the class.
     *
     * @return the required Class object
     */
    protected Class
    loadClass(String name, boolean resolve)
    throws ClassNotFoundException
    {
        // First, check if the class has already been loaded
        // System.out.println("findLoadedClass " + name);
        Class c = findLoadedClass(name);
        if (c == null) {
            try {
                // If still not found, then call findClass in order
                // to find the class.
                // System.out.println("findClass " + name);
                c = findClass(name);
            } catch (ClassNotFoundException e) {
                // System.out.println("loadClass " + name);
                return super.loadClass(name, resolve);
            }
        }
        if (resolve) {
            resolveClass(c);
        }
        return c;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Searches for and load a class on the classpath of this class loader.
     *
     * @param name - The name of the class to be loaded. Must not be null.
     *
     * @return The required Class object
     *
     * @exception ClassNotFoundException if the requested class does not exist
     *                                   on this loader's classpath.
     */
    public Class
    findClass(String name)
    throws ClassNotFoundException
    {
        // System.out.println("Call findClass for "+name);
        return super.findClass(name);
    }

    /**
     * Loads the class with the specified name.
     *
     * @param name - The name of the class to be loaded. Must not be null.
     *
     * @return the required Class object
     */
    public Class
    loadClass(String name)
    throws ClassNotFoundException
    {
        //System.out.println("LOAD CLASS "+name);
        return loadClass(name, false);
    }

    /**
     * Adds URL to class loader.
     *
     * @param filename - A valid URL to load.
     * Must not be null.
     */
    public void
    addURL(java.net.URL url)
    {
        super.addURL(url);
    }
}
